@@ -18,17 +18,16 @@ package org.apache.qpid.jms;
 
 import java.io.IOException;
 import java.net.URI;
-import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CopyOnWriteArraySet;
 import java.util.concurrent.LinkedBlockingQueue;
-import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicLong;
+import java.util.concurrent.atomic.AtomicReference;
 
 import javax.jms.Connection;
 import javax.jms.ConnectionConsumer;
@@ -80,6 +79,7 @@ import org.apache.qpid.jms.provider.ProviderConstants.ACK_TYPE;
 import org.apache.qpid.jms.provider.ProviderFuture;
 import org.apache.qpid.jms.provider.ProviderListener;
 import org.apache.qpid.jms.provider.ProviderSynchronization;
+import org.apache.qpid.jms.util.QpidJMSThreadFactory;
 import org.apache.qpid.jms.util.ThreadPoolUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -96,11 +96,10 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
     private final AtomicBoolean closed = new AtomicBoolean();
     private final AtomicBoolean closing = new AtomicBoolean();
     private final AtomicBoolean started = new AtomicBoolean();
-    private final AtomicBoolean failed = new AtomicBoolean();
+    private final AtomicReference<IOException> failureCause = new AtomicReference<>();
     private final JmsConnectionInfo connectionInfo;
     private final ThreadPoolExecutor executor;
 
-    private volatile IOException firstFailureError;
     private ExceptionListener exceptionListener;
     private JmsMessageFactory messageFactory;
     private Provider provider;
@@ -122,14 +121,10 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
         // will also serve as a means of preventing JVM shutdown should a client application
         // not have it's own mechanism for doing so if the configuration specifies that the
         // Connection create this thread as a non-daemon thread.
-        executor = new ThreadPoolExecutor(1, 1, 5, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(), new ThreadFactory() {
-            @Override
-            public Thread newThread(Runnable target) {
-                Thread thread = new Thread(target, "QpidJMS Connection Executor: " + connectionInfo.getId());
-                thread.setDaemon(connectionInfo.isUseDaemonThread());
-                return thread;
-            }
-        });
+        executor = new ThreadPoolExecutor(1, 1, 5, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),
+            new QpidJMSThreadFactory("QpidJMS Connection Executor: " + connectionInfo.getId(), connectionInfo.isUseDaemonThread()));
+
+        executor.setRejectedExecutionHandler(new ThreadPoolExecutor.DiscardOldestPolicy());
 
         // We need to start the core thread in order for it to prevent JVM shutdown as our
         // single non-daemon thread.
@@ -190,7 +185,7 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
 
         try {
 
-            if (!closed.get() && !failed.get()) {
+            if (!closed.get() && !isFailed()) {
                 // do not fail if already closed as specified by the JMS specification.
                 doStop(false);
             }
@@ -207,7 +202,7 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
                     session.shutdown();
                 }
 
-                if (isConnected() && !failed.get()) {
+                if (isConnected() && !isFailed()) {
                     ProviderFuture request = new ProviderFuture();
                     requests.put(request, request);
                     try {
@@ -276,7 +271,7 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
             session.shutdown(cause);
         }
 
-        if (isConnected() && !failed.get() && !closing.get()) {
+        if (isConnected() && !isFailed() && !closing.get()) {
             destroyResource(connectionInfo);
         }
 
@@ -558,8 +553,8 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
 
     protected void checkClosedOrFailed() throws JMSException {
         checkClosed();
-        if (failed.get()) {
-            throw new JmsConnectionFailedException(firstFailureError);
+        if (failureCause.get() != null) {
+            throw new JmsConnectionFailedException(failureCause.get());
         }
     }
 
@@ -1033,7 +1028,7 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
     }
 
     public boolean isFailed() {
-        return failed.get();
+        return failureCause.get() != null;
     }
 
     public JmsConnectionId getId() {
@@ -1374,26 +1369,6 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
         onAsyncException(cause);
     }
 
-    @Override
-    public void onRemoteDiscovery(final List<URI> remotes) {
-        for (URI remote : remotes) {
-            LOG.trace("Discovered new remote at: {}", remote);
-        }
-
-        // Give listeners a chance to know what we've discovered.
-        if (!connectionListeners.isEmpty()) {
-            for (final JmsConnectionListener listener : connectionListeners) {
-                executor.submit(new Runnable() {
-
-                    @Override
-                    public void run() {
-                        listener.onRemoteDiscovery(remotes);
-                    }
-                });
-            }
-        }
-    }
-
     /**
      * Handles any asynchronous errors that occur from the JMS framework classes.
      *
@@ -1421,10 +1396,7 @@ public class JmsConnection implements AutoCloseable, Connection, TopicConnection
         }
     }
 
-    protected void providerFailed(IOException error) {
-        failed.set(true);
-        if (firstFailureError == null) {
-            firstFailureError = error;
-        }
+    protected void providerFailed(IOException cause) {
+        failureCause.compareAndSet(null, cause);
     }
 }
