@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -30,6 +30,7 @@ import javax.jms.Connection;
 import javax.jms.DeliveryMode;
 import javax.jms.ExceptionListener;
 import javax.jms.JMSException;
+import javax.jms.JMSSecurityException;
 import javax.jms.Message;
 import javax.jms.MessageConsumer;
 import javax.jms.MessageProducer;
@@ -38,7 +39,6 @@ import javax.jms.Session;
 import javax.jms.Topic;
 
 import org.apache.activemq.broker.jmx.QueueViewMBean;
-import org.apache.qpid.jms.JmsConnection;
 import org.apache.qpid.jms.JmsConnectionFactory;
 import org.apache.qpid.jms.support.AmqpTestSupport;
 import org.apache.qpid.jms.support.Wait;
@@ -71,6 +71,21 @@ public class JmsFailoverTest extends AmqpTestSupport {
         connection.close();
     }
 
+    @Test(timeout=30000, expected = JMSSecurityException.class)
+    public void testCreateConnectionAsUnknwonUser() throws Exception {
+        URI brokerURI = new URI(getAmqpFailoverURI() +
+            "?failover.maxReconnectAttempts=5");
+
+        JmsConnectionFactory factory = new JmsConnectionFactory(brokerURI);
+        factory.setUsername("unknown");
+        factory.setPassword("unknown");
+
+        connection = factory.createConnection();
+        assertNotNull(connection);
+        connection.start();
+        connection.close();
+    }
+
     @Test(timeout=60000)
     public void testStartupReconnectAttempts() throws Exception {
         URI brokerURI = new URI("failover://(amqp://localhost:5677)" +
@@ -323,7 +338,7 @@ public class JmsFailoverTest extends AmqpTestSupport {
 
     @Test(timeout = 30000)
     public void testPullConsumerTimedReceiveRecovers() throws Exception {
-        URI brokerURI = new URI(getAmqpFailoverURI());
+        URI brokerURI = new URI(getAmqpFailoverURI() + "?jms.prefetchPolicy.all=0");
 
         final CountDownLatch started = new CountDownLatch(1);
         final CountDownLatch received = new CountDownLatch(1);
@@ -331,10 +346,6 @@ public class JmsFailoverTest extends AmqpTestSupport {
         connection = createAmqpConnection(brokerURI);
         connection.start();
 
-        // Make all our consumers pull consumers.
-        JmsConnection jmsConnection = (JmsConnection) connection;
-        jmsConnection.getPrefetchPolicy().setAll(0);
-
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         Queue queue = session.createQueue(name.getMethodName());
         final MessageConsumer consumer = session.createConsumer(queue);
@@ -345,7 +356,7 @@ public class JmsFailoverTest extends AmqpTestSupport {
             public void run() {
                 started.countDown();
                 try {
-                    Message message = consumer.receive(30000);
+                    Message message = consumer.receive(10000);
                     if (message != null) {
                         received.countDown();
                     } else {
@@ -387,7 +398,7 @@ public class JmsFailoverTest extends AmqpTestSupport {
 
     @Test(timeout = 30000)
     public void testPullConsumerReceiveRecovers() throws Exception {
-        URI brokerURI = new URI(getAmqpFailoverURI());
+        URI brokerURI = new URI(getAmqpFailoverURI() + "?jms.prefetchPolicy.all=0");
 
         final CountDownLatch started = new CountDownLatch(1);
         final CountDownLatch received = new CountDownLatch(1);
@@ -395,10 +406,6 @@ public class JmsFailoverTest extends AmqpTestSupport {
         connection = createAmqpConnection(brokerURI);
         connection.start();
 
-        // Make all our consumers pull consumers.
-        JmsConnection jmsConnection = (JmsConnection) connection;
-        jmsConnection.getPrefetchPolicy().setAll(0);
-
         Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
         Queue queue = session.createQueue(name.getMethodName());
         final MessageConsumer consumer = session.createConsumer(queue);
