@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -20,11 +20,13 @@ import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.util.Arrays;
 import java.util.concurrent.TimeUnit;
 
 import javax.jms.BytesMessage;
+import javax.jms.Connection;
 import javax.jms.DeliveryMode;
 import javax.jms.JMSSecurityException;
 import javax.jms.Message;
@@ -60,6 +62,19 @@ public class JmsMessageProducerTest extends AmqpTestSupport {
         assertEquals(0, proxy.getQueueSize());
     }
 
+    @Test(timeout = 30000)
+    public void testProducerNotAuthorized() throws Exception {
+        connection = createAmqpConnection("guest", "password");
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        Queue queue = session.createQueue("USERS.txQueue");
+        try {
+            session.createProducer(queue);
+            fail("Should not be able to produce here.");
+        } catch (JMSSecurityException jmsSE) {
+            LOG.info("Caught expected exception");
+        }
+    }
+
     @Test(timeout = 20000)
     public void testSendMultipleMessagesPersistent() throws Exception {
         connection = createAmqpConnection();
@@ -132,46 +147,6 @@ public class JmsMessageProducerTest extends AmqpTestSupport {
         producer.close();
     }
 
-    @Test(timeout = 20000)
-    public void testSendWorksWhenConnectionNotStarted() throws Exception {
-        connection = createAmqpConnection();
-        assertNotNull(connection);
-
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        assertNotNull(session);
-        Queue queue = session.createQueue(name.getMethodName());
-        MessageProducer producer = session.createProducer(queue);
-
-        QueueViewMBean proxy = getProxyToQueue(name.getMethodName());
-        assertEquals(0, proxy.getQueueSize());
-
-        Message message = session.createMessage();
-        producer.send(message);
-
-        assertEquals(1, proxy.getQueueSize());
-    }
-
-    @Test(timeout = 20000)
-    public void testSendWorksAfterConnectionStopped() throws Exception {
-        connection = createAmqpConnection();
-        assertNotNull(connection);
-        connection.start();
-
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        assertNotNull(session);
-        Queue queue = session.createQueue(name.getMethodName());
-        MessageProducer producer = session.createProducer(queue);
-
-        QueueViewMBean proxy = getProxyToQueue(name.getMethodName());
-        assertEquals(0, proxy.getQueueSize());
-        connection.stop();
-
-        Message message = session.createMessage();
-        producer.send(message);
-
-        assertEquals(1, proxy.getQueueSize());
-    }
-
     @Test(timeout = 20000)
     public void testPersistentSendsAreMarkedPersistent() throws Exception {
         connection = createAmqpConnection();
@@ -198,6 +173,43 @@ public class JmsMessageProducerTest extends AmqpTestSupport {
         assertTrue(message.getJMSDeliveryMode() == DeliveryMode.PERSISTENT);
     }
 
+    @Test(timeout = 20000)
+    public void testSendForeignMessage() throws Exception {
+        connection = createAmqpConnection();
+        assertNotNull(connection);
+        connection.start();
+
+        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+        assertNotNull(session);
+        Queue queue = session.createQueue(name.getMethodName());
+        MessageProducer producer = session.createProducer(queue);
+
+        QueueViewMBean proxy = getProxyToQueue(name.getMethodName());
+        assertEquals(0, proxy.getQueueSize());
+
+        String foreignPropKey = "myForeignMessageProp";
+        String foreignPropValue = "ABC456XYZ";
+
+        Connection activeMQConnection = createActiveMQConnection();
+        try {
+            Session activeMQSession = activeMQConnection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+
+            Message foreignMessage = activeMQSession.createMessage();
+            foreignMessage.setStringProperty(foreignPropKey, foreignPropValue);
+
+            producer.send(foreignMessage);
+        } finally {
+            activeMQConnection.close();
+        }
+
+        assertEquals(1, proxy.getQueueSize());
+
+        MessageConsumer consumer = session.createConsumer(queue);
+        Message receivedMessage = consumer.receive(5000);
+        assertNotNull("Did not receive message as expected", receivedMessage);
+        assertEquals("Unexpected property value", foreignPropValue, receivedMessage.getStringProperty(foreignPropKey));
+    }
+
     @Test(timeout = 20000)
     public void testProducerWithNoTTLSendsMessagesWithoutTTL() throws Exception {
         connection = createAmqpConnection();
@@ -291,15 +303,6 @@ public class JmsMessageProducerTest extends AmqpTestSupport {
         assertEquals(messageText, textMessage.getText());
     }
 
-    @Test(timeout=90000, expected=JMSSecurityException.class)
-    public void testProducerNotAuthorized() throws Exception{
-        connection = createAmqpConnection("guest", "password");
-        connection.start();
-        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
-        Queue queue = session.createQueue("USERS." + name.getMethodName());
-        session.createProducer(queue);
-    }
-
     @Test(timeout = 20000)
     public void testProducerWithTTL() throws Exception {
         doProducerWithTTLTestImpl(false, null);
